package finger

import (
	"context"
	"fmt"
	"io"
	"net"

	"tildegit.org/tjp/sliderule/internal/types"
	"tildegit.org/tjp/sliderule/internal"
	"tildegit.org/tjp/sliderule/logging"
)

type fingerServer struct {
	internal.Server
	handler types.Handler
}

func (fs fingerServer) Protocol() string { return "FINGER" }

// NewServer builds a finger server.
func NewServer(
	ctx context.Context,
	hostname string,
	network string,
	address string,
	handler types.Handler,
	errLog logging.Logger,
) (types.Server, error) {
	fs := &fingerServer{handler: handler}

	hostname = internal.JoinDefaultPort(hostname, "79")
	address = internal.JoinDefaultPort(address, "79")

	var err error
	fs.Server, err = internal.NewServer(ctx, hostname, network, address, errLog, fs.handleConn)
	if err != nil {
		return nil, err
	}

	return fs, nil
}

func (fs *fingerServer) handleConn(conn net.Conn) {
	request, err := ParseRequest(conn)
	if err != nil {
		_, _ = fmt.Fprint(conn, err.Error()+"\r\n")
		return
	}

	request.Server = fs
	request.RemoteAddr = conn.RemoteAddr()

	defer func() {
		if r := recover(); r != nil {
			_ = fs.LogError("msg", "panic in handler", "err", r)
			_, _ = fmt.Fprint(conn, "Error handling request.\r\n")
		}
	}()
	response := fs.handler.Handle(fs.Ctx, request)
	if response == nil {
		response = Error("No result found.")
	}

	defer response.Close()
	_, _ = io.Copy(conn, response.Body)
}
