package htmlconv_test

import (
	"bytes"
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"

	"tildegit.org/tjp/sliderule/gemini/gemtext"
	"tildegit.org/tjp/sliderule/gemini/gemtext/htmlconv"
)

var gmiDoc = `
# top-level header line

## subtitle

This is some non-blank regular text.

* an
* unordered
* list

=> gemini://google.com/ as if
=> https://google.com/

> this is a quote
> -tjp

`[1:] + "```pre-formatted code\ndoc := gemtext.Parse(req.Body)\n```ignored closing alt-text\n"

func TestConvert(t *testing.T) {
	htmlDoc := `
<!DOCTYPE html>
<html><body class="gemtext"><h1 class="gemtext">top-level header line</h1><h2 class="gemtext">subtitle</h2><p class="gemtext">This is some non-blank regular text.
</p><ul class="gemtext"><li class="gemtext">an</li><li class="gemtext">unordered</li><li class="gemtext">list</li></ul><p class="gemtext">=> <a class="gemtext" href="gemini://google.com/">as if</a></p><p class="gemtext">=> <a class="gemtext" href="https://google.com/">https://google.com/</a></p><blockquote class="gemtext"> this is a quote</blockquote><blockquote class="gemtext"> -tjp</blockquote><pre class="gemtext">doc := gemtext.Parse(req.Body)
</pre></body></html>`[1:]

	doc, err := gemtext.Parse(bytes.NewBufferString(gmiDoc))
	require.Nil(t, err)

	buf := &bytes.Buffer{}
	require.Nil(t, htmlconv.Convert(buf, doc, nil))

	assert.Equal(t, htmlDoc, buf.String())
}
